]> diplodocus.org Git - nmh/blobdiff - uip/new.c
Alter HasSuffixC()'s char * to be const.
[nmh] / uip / new.c
index 74d4d5122a91dcadecf65304eacbf9deaf831f45..ff9c398bb7a0646cf805ba969ae11e88bda7c86e 100644 (file)
--- a/uip/new.c
+++ b/uip/new.c
@@ -31,7 +31,13 @@ DEFINE_SWITCH_ENUM(NEW);
 DEFINE_SWITCH_ARRAY(NEW, switches);
 #undef X
 
 DEFINE_SWITCH_ARRAY(NEW, switches);
 #undef X
 
-static enum { NEW, FNEXT, FPREV, UNSEEN } run_mode = NEW;
+/* What to do, based on argv[0]. */
+static enum {
+    RM_NEW,
+    RM_FNEXT,
+    RM_FPREV,
+    RM_UNSEEN
+} run_mode = RM_NEW;
 
 /* check_folders uses this to maintain state with both .folders list of
  * folders and with crawl_folders. */
 
 /* check_folders uses this to maintain state with both .folders list of
  * folders and with crawl_folders. */
@@ -95,6 +101,7 @@ get_msgnums(char *folder, char *sequences[])
     char name[NAMESZ], field[BUFSIZ];
     char *cp;
     char *msgnums = NULL, *this_msgnums, *old_msgnums;
     char name[NAMESZ], field[BUFSIZ];
     char *cp;
     char *msgnums = NULL, *this_msgnums, *old_msgnums;
+    int failed_to_lock = 0;
     m_getfld_state_t gstate = 0;
 
     /* copied from seq_read.c:seq_public */
     m_getfld_state_t gstate = 0;
 
     /* copied from seq_read.c:seq_public */
@@ -112,9 +119,14 @@ get_msgnums(char *folder, char *sequences[])
     if (seqfile == NULL)
        return NULL;
 
     if (seqfile == NULL)
        return NULL;
 
-    if ((fp = lkfopendata (seqfile, "r")) == NULL) {
-       free(seqfile);
-       return NULL;
+    if ((fp = lkfopendata (seqfile, "r", & failed_to_lock)) == NULL) {
+
+       if (failed_to_lock) {
+           adios (seqfile, "failed to lock");
+       } else {
+           free(seqfile);
+           return NULL;
+       }
     }
 
     /* Use m_getfld to scan sequence file */
     }
 
     /* Use m_getfld to scan sequence file */
@@ -195,9 +207,10 @@ check_folder(char *folder, size_t len, struct list_state *b)
 
     if (is_cur || msgnums != NULL) {
        if (*b->first == NULL) {
 
     if (is_cur || msgnums != NULL) {
        if (*b->first == NULL) {
-           *b->first = b->node = mh_xmalloc(sizeof(*b->node));
+           NEW(b->node);
+           *b->first = b->node;
        } else {
        } else {
-           b->node->n_next = mh_xmalloc(sizeof(*b->node));
+           NEW(b->node->n_next);
            b->node = b->node->n_next;
        }
        b->node->n_name = folder;
            b->node = b->node->n_next;
        }
        b->node->n_name = folder;
@@ -255,7 +268,9 @@ check_folders(struct node **first, struct node **last,
     b.sequences = sequences;
 
     if (folders == NULL) {
     b.sequences = sequences;
 
     if (folders == NULL) {
-       chdir(m_maildir(""));
+       if (chdir(m_maildir("")) < 0) {
+           advise (m_maildir(""), "chdir");
+       }
        crawl_folders(".", crawl_callback, &b);
     } else {
        fp = fopen(folders, "r");
        crawl_folders(".", crawl_callback, &b);
     } else {
        fp = fopen(folders, "r");
@@ -265,7 +280,7 @@ check_folders(struct node **first, struct node **last,
        while (vfgets(fp, &line) == OK) {
            len = strlen(line) - 1;
            line[len] = '\0';
        while (vfgets(fp, &line) == OK) {
            len = strlen(line) - 1;
            line[len] = '\0';
-           check_folder(getcpy(line), len, &b);
+           check_folder(mh_xstrdup(line), len, &b);
        }
        fclose(fp);
     }
        }
        fclose(fp);
     }
@@ -301,8 +316,8 @@ join_sequences(char *sequences[])
 }
 
 /* Return a struct node for the folder to change to.  This is the next
 }
 
 /* Return a struct node for the folder to change to.  This is the next
* (previous, if FPREV mode) folder with desired messages, or the current
* folder if no folders have desired.  If NEW or UNSEEN mode, print the
  * output but don't change folders.
  *
  * n_name is the folder to change to, and n_field is the string list of
  * output but don't change folders.
  *
  * n_name is the folder to change to, and n_field is the string list of
@@ -323,35 +338,36 @@ doit(char *cur, char *folders, char *sequences[])
     check_folders(&first, &last, &cur_node, &folder_len, cur,
                  folders, sequences);
 
     check_folders(&first, &last, &cur_node, &folder_len, cur,
                  folders, sequences);
 
-    if (run_mode == FNEXT || run_mode == FPREV) {
+    if (run_mode == RM_FNEXT || run_mode == RM_FPREV) {
        if (first == NULL) {
            /* No folders at all... */
            return NULL;
        if (first == NULL) {
            /* No folders at all... */
            return NULL;
-       } else if (first->n_next == NULL) {
+       }
+        if (first->n_next == NULL) {
            /* We have only one node; any desired messages in it? */
            if (first->n_field == NULL) {
                return NULL;
            /* We have only one node; any desired messages in it? */
            if (first->n_field == NULL) {
                return NULL;
-           } else {
-               return first;
            }
            }
-       } else if (cur_node == NULL) {
+            return first;
+       }
+        if (cur_node == NULL) {
            /* Current folder is not listed in .folders, return first. */
            return first;
        }
            /* Current folder is not listed in .folders, return first. */
            return first;
        }
-    } else if (run_mode == UNSEEN) {
+    } else if (run_mode == RM_UNSEEN) {
        sequences_s = join_sequences(sequences);
     }
 
     for (node = first, prev = NULL;
         node != NULL;
         prev = node, node = node->n_next) {
        sequences_s = join_sequences(sequences);
     }
 
     for (node = first, prev = NULL;
         node != NULL;
         prev = node, node = node->n_next) {
-        if (run_mode == FNEXT) {
+        if (run_mode == RM_FNEXT) {
             /* If we have a previous node and it is the current
              * folder, return this node. */
             if (prev != NULL && strcmp(prev->n_name, cur) == 0) {
                 return node;
             }
             /* If we have a previous node and it is the current
              * folder, return this node. */
             if (prev != NULL && strcmp(prev->n_name, cur) == 0) {
                 return node;
             }
-        } else if (run_mode == FPREV) {
+        } else if (run_mode == RM_FPREV) {
             if (strcmp(node->n_name, cur) == 0) {
                 /* Found current folder in fprev mode; if we have a
                  * previous node in the list, return it; else return
             if (strcmp(node->n_name, cur) == 0) {
                 /* Found current folder in fprev mode; if we have a
                  * previous node in the list, return it; else return
@@ -361,7 +377,9 @@ doit(char *cur, char *folders, char *sequences[])
                 }
                 return prev;
             }
                 }
                 return prev;
             }
-        } else if (run_mode == UNSEEN) {
+        } else if (run_mode == RM_UNSEEN) {
+            int status;
+
             if (node->n_field == NULL) {
                 continue;
             }
             if (node->n_field == NULL) {
                 continue;
             }
@@ -380,7 +398,10 @@ doit(char *cur, char *folders, char *sequences[])
            /* TODO: Split enough of scan.c out so that we can call it here. */
            command = concat("scan +", node->n_name, " ", sequences_s,
                             (void *)NULL);
            /* TODO: Split enough of scan.c out so that we can call it here. */
            command = concat("scan +", node->n_name, " ", sequences_s,
                             (void *)NULL);
-           system(command);
+           status = system(command);
+           if (! WIFEXITED (status)) {
+               adios (command, "system");
+           }
            free(command);
         } else {
             if (node->n_field == NULL) {
            free(command);
         } else {
             if (node->n_field == NULL) {
@@ -400,11 +421,14 @@ doit(char *cur, char *folders, char *sequences[])
 
     /* If we're fnext, we haven't checked the last node yet.  If it's the
      * current folder, return the first node. */
 
     /* If we're fnext, we haven't checked the last node yet.  If it's the
      * current folder, return the first node. */
-    if (run_mode == FNEXT && strcmp(last->n_name, cur) == 0) {
-        return first;
+    if (run_mode == RM_FNEXT) {
+       assert(last != NULL);
+       if (strcmp(last->n_name, cur) == 0) {
+            return first;
+       }
     }
 
     }
 
-    if (run_mode == NEW) {
+    if (run_mode == RM_NEW) {
         printf("%-*s %6d.\n", (int) folder_len, " total", total);
     }
 
         printf("%-*s %6d.\n", (int) folder_len, " total", total);
     }
 
@@ -422,13 +446,7 @@ main(int argc, char **argv)
     char *unseen;
     struct node *folder;
 
     char *unseen;
     struct node *folder;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex(argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -473,11 +491,11 @@ main(int argc, char **argv)
     }
 
     if (strcmp(invo_name, "fnext") == 0) {
     }
 
     if (strcmp(invo_name, "fnext") == 0) {
-        run_mode = FNEXT;
+        run_mode = RM_FNEXT;
     } else if (strcmp(invo_name, "fprev") == 0) {
     } else if (strcmp(invo_name, "fprev") == 0) {
-        run_mode = FPREV;
+        run_mode = RM_FPREV;
     } else if (strcmp(invo_name, "unseen") == 0) {
     } else if (strcmp(invo_name, "unseen") == 0) {
-        run_mode = UNSEEN;
+        run_mode = RM_UNSEEN;
     }
 
     if (folders == NULL) {
     }
 
     if (folders == NULL) {
@@ -506,7 +524,7 @@ main(int argc, char **argv)
         return 1;
     }
 
         return 1;
     }
 
-    if (run_mode == UNSEEN) {
+    if (run_mode == RM_UNSEEN) {
         /* All the scan(1)s it runs change the current folder, so we
          * need to put it back.  Unfortunately, context_replace lamely
          * ignores the new value you give it if it is the same one it
         /* All the scan(1)s it runs change the current folder, so we
          * need to put it back.  Unfortunately, context_replace lamely
          * ignores the new value you give it if it is the same one it
@@ -518,7 +536,7 @@ main(int argc, char **argv)
     /* update current folder */
     context_replace(pfolder, folder->n_name);
 
     /* update current folder */
     context_replace(pfolder, folder->n_name);
 
-    if (run_mode == FNEXT || run_mode == FPREV) {
+    if (run_mode == RM_FNEXT || run_mode == RM_FPREV) {
         printf("%s  %s\n", folder->n_name, folder->n_field);
     }
 
@@ -301,8 +316,8 @@ join_sequences(char *sequences[])
 
 /* Return a struct node for the folder to change to.  This is the next
- * (previous, if FPREV mode) folder with desired messages, or the current
- * folder if no folders have desired.  If NEW or UNSEEN mode, print the
+ * (previous, if RM_FPREV mode) folder with desired messages, or the current
+ * folder if no folders have desired.  If RM_NEW or RM_UNSEEN mode, print the
  * output but don't change folders.
  *
         printf("%s  %s\n", folder->n_name, folder->n_field);
     }